from __future__ import annotations

from dataclasses import dataclass


def to_camel(s: str):
    parts = iter(s.split("_"))
    return next(parts) + "".join(i.title() for i in parts)


@dataclass
class BaseLayer:
    type: str
    # data: Any | None = None
    # id: str | None = None

    # def __init__(
    #     self,
    #     data: Any | None = None,
    #     id: str | None = None,
    #     **kwargs,
    # ):
    #     self.id = id
    #     self.data = data
    #     self._kwargs = kwargs



from __future__ import annotations

import json
from dataclasses import dataclass
from typing import Literal

from .base import BaseLayer


@dataclass
class CityJSONLayer(BaseLayer):
    format: Literal["cityjson", "cityjsonseq"]

    type = "CityJSONLayer"

    def __init__(
        self,
        id: str | None = None,
        *,
        data: str | None = None,
        format: Literal["cityjson", "cityjsonseq"] = "cityjson",
        get_fill_color: str | None = None,
    ):
        self.id = id
        self.data = data
        self.format = format
        self.get_fill_color = get_fill_color

    def serialize(self):
        return {
            "id": self.id,
            "format": self.format,
            "data": self.data,
            "getFillColor": self.get_fill_color,
        }



from __future__ import annotations

from logging import getLogger
from pathlib import Path
from typing import Literal

import anywidget
import traitlets

logger = getLogger(__name__)


class BaseView(anywidget.AnyWidget):
    _esm = Path(__file__).parent / "static/widget.js"
    _css = Path(__file__).parent / "static/widget.css"

    mode = traitlets.Unicode().tag(sync=True)

    theme = traitlets.Unicode("light").tag(sync=True)

    width = traitlets.Union([traitlets.Unicode(), traitlets.Int()]).tag(sync=True)
    height = traitlets.Union([traitlets.Unicode(), traitlets.Int()]).tag(sync=True)

    data = traitlets.Unicode().tag(sync=True)
    click = traitlets.Dict(allow_none=True).tag(sync=True)

    layers = traitlets.List()
    _layers = traitlets.List().tag(sync=True)


class MapView(BaseView):
    map_style = traitlets.Unicode().tag(sync=True)
    map_provider = traitlets.Unicode("carto").tag(sync=True)

    def __init__(
        self,
        *,
        # layers: List[Layer] = [],
        # initial_view_state: ViewState = ViewState(latitude=0, longitude=0, zoom=1),
        width: int | str = "100%",
        height: int | str = 500,
        theme: Literal["light", "dark"] = "dark",
        map_style=None,
        map_provider="carto",
        api_keys=None,
    ):
        # self.inital_view_state = initial_view_state
        # self.layers = layers
        self.mode = "map"

        self.theme = theme

        self.map_style = map_style or theme
        self.map_provider = map_provider

        self.width = width
        self.height = height

        # self._tooltip = tooltip

        # if map_style in DefaultBaseMap.short_names:
        #     map_style = DefaultBaseMap.short_names[map_style]
        # self.map_style = map_style

        # self._initial_view_state = asdict(initial_view_state)
        # self._layers = list(map(lambda x: x.serialize(), layers))

        super().__init__()

    def update(self):
        self._layers = list(map(lambda x: x.serialize(), self.layers))


class VirtualView(BaseView):
    def __init__(
        self,
        *,
        # layers: List[Layer] = [],
        # initial_view_state: ViewState = ViewState(latitude=0, longitude=0, zoom=1),
        theme: Literal["light", "dark"] = "light",
        width: int | str = "100%",
        height: int | str = 500,
    ):
        # self.inital_view_state = initial_view_state
        # self.layers = layers
        self.mode = "virtual"

        self.theme = theme

        self.width = width
        self.height = height
        # self._tooltip = tooltip

        # self._initial_view_state = asdict(initial_view_state)
        # self._layers = list(map(lambda x: x.serialize(), layers))

        super().__init__()

    def update(self):
        self._layers = list(map(lambda x: x.serialize(), self.layers))

    # def update(self, data: str):
    #     self.data = data



import importlib.metadata

from .layers.cityjson import CityJSONLayer
from .view import MapView, VirtualView

try:
    __version__ = importlib.metadata.version("cityview")
except importlib.metadata.PackageNotFoundError:
    __version__ = "unknown"

__all__ = ["MapView", "VirtualView", "CityJSONLayer"]
